@@ -7,20 +7,16 @@ from llmling.resources.loaders import (
     PathResourceLoader,
     SourceResourceLoader,
     TextResourceLoader,
+    RepositoryResourceLoader,
+    ImageResourceLoader,
 )
 from llmling.resources.loaders.registry import ResourceLoaderRegistry
 from llmling.resources.registry import ResourceRegistry
 from llmling.resources.models import LoadedResource
-from llmling.resources.loaders.image import ImageResourceLoader
 
 # Create and populate the default registry
 default_registry = ResourceLoaderRegistry()
-default_registry["image"] = ImageResourceLoader
-default_registry["path"] = PathResourceLoader
-default_registry["text"] = TextResourceLoader
-default_registry["cli"] = CLIResourceLoader
-default_registry["source"] = SourceResourceLoader
-default_registry["callable"] = CallableResourceLoader
+default_registry.register_default_loaders()
 
 __all__ = [
     "CLIResourceLoader",
@@ -28,6 +24,7 @@ __all__ = [
     "ImageResourceLoader",
     "LoadedResource",
     "PathResourceLoader",
+    "RepositoryResourceLoader",
     "ResourceLoader",
     "ResourceLoaderRegistry",
     "ResourceRegistry",
