@@ -5,19 +5,24 @@ from __future__ import annotations
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from datetime import datetime
+import os
+import re
 from typing import TYPE_CHECKING, Any, ClassVar, TypeVar, cast, overload
+import urllib.parse
 
-import logfire
-
+from llmling.completions.protocols import CompletionProvider
 from llmling.config.models import BaseResource
 from llmling.core import exceptions
 from llmling.core.descriptors import classproperty
 from llmling.core.log import get_logger
 from llmling.core.typedefs import MessageContent, MessageContentType
 from llmling.resources.models import LoadedResource, ResourceMetadata
+from llmling.utils import paths
 
 
 if TYPE_CHECKING:
+    from collections.abc import AsyncIterator
+
     from llmling.processors.registry import ProcessorRegistry
 
 
@@ -91,17 +96,28 @@ class LoaderContext[TResource]:
         return f"{cls_name}(name={self.name!r}, type={resource.resource_type})"
 
 
-class ResourceLoader[TResource](ABC):
+class ResourceLoader[TResource](ABC, CompletionProvider):
     """Base class for resource loaders."""
 
     context_class: type[TResource]
     uri_scheme: ClassVar[str]
     supported_mime_types: ClassVar[list[str]] = ["text/plain"]
+    # Invalid path characters (Windows + Unix)
+    invalid_chars_pattern = re.compile(r'[\x00-\x1F<>:"|?*\\]')
 
     def __init__(self, context: LoaderContext[TResource] | None = None) -> None:
         """Initialize loader with optional context."""
         self.context = context
 
+    async def get_completions(
+        self,
+        current_value: str,
+        argument_name: str | None = None,
+        **options: Any,
+    ) -> list[str]:
+        """Get completions for this resource type. Override to implement."""
+        return []
+
     @classmethod
     def create(cls, resource: TResource, name: str) -> ResourceLoader[TResource]:
         """Create a loader instance with named context."""
@@ -119,10 +135,10 @@ class ResourceLoader[TResource](ABC):
 
     @classmethod
     def get_name_from_uri(cls, uri: str) -> str:
-        """Extract resource name from URI.
+        """Extract resource name from URI, ignoring parameters.
 
         Args:
-            uri: URI in format "{scheme}://{name}"
+            uri: URI in format "{scheme}://{name}[?param1=value1]"
 
         Returns:
             Resource name
@@ -132,34 +148,73 @@ class ResourceLoader[TResource](ABC):
         """
         try:
             if not cls.supports_uri(uri):
-                msg = f"Unsupported URI scheme: {uri}"
+                msg = f"Unsupported URI: {uri}"
                 raise exceptions.LoaderError(msg)  # noqa: TRY301
 
-            # Split "scheme://name" and get name part
-            _, name = uri.split("://", 1)
+            parsed = urllib.parse.urlparse(uri)
+            path = parsed.netloc or parsed.path.lstrip("/")
 
-            # Normalize path separators
-            return name.replace("\\", "/").lstrip("/")
+            # Get parts excluding protocol info
+            parts = [
+                urllib.parse.unquote(str(part))  # URL decode each part
+                for part in path.split("/")
+                if not paths.is_ignorable_part(str(part))
+            ]
 
+            if not parts:
+                msg = "Empty path after normalization"
+                raise exceptions.LoaderError(msg)  # noqa: TRY301
+
+            # Validate path components
+            if any(cls.invalid_chars_pattern.search(part) for part in parts):
+                msg = "Invalid characters in path component"
+                raise exceptions.LoaderError(msg)  # noqa: TRY301
+
+            # Join with forward slashes and normalize path
+            joined = "/".join(parts)
+            # Normalize path (resolve .. and .)
+            normalized = os.path.normpath(joined).replace("\\", "/")
         except Exception as exc:
+            if isinstance(exc, exceptions.LoaderError):
+                raise
             msg = f"Invalid URI format: {uri}"
             raise exceptions.LoaderError(msg) from exc
+        else:
+            return normalized
 
     @classmethod
-    def create_uri(cls, *, name: str) -> str:
+    def get_params_from_uri(cls, uri: str) -> dict[str, str]:
+        """Extract parameters from URI.
+
+        Args:
+            uri: URI containing query parameters
+
+        Returns:
+            Dictionary of parameters
+        """
+        parsed = urllib.parse.urlparse(uri)
+        return dict(urllib.parse.parse_qsl(parsed.query))
+
+    def create_uri(self, *, name: str, params: dict[str, str] | None = None) -> str:
         """Create a valid URI for this resource type.
 
         Args:
             name: Resource name or identifier
+            params: Optional URI parameters
 
         Returns:
-            URI in format "{scheme}://{name}"
+            URI in format "{scheme}://{name}[?param1=value1&...]"
         """
         # Remove any existing scheme
         if "://" in name:
             name = name.split("://", 1)[1]
 
-        return cls.get_uri_template().format(name=name)
+        base_uri = self.get_uri_template().format(name=name)
+        if not params:
+            return base_uri
+
+        query = urllib.parse.urlencode(params)
+        return f"{base_uri}?{query}"
 
     def __repr__(self) -> str:
         """Show loader type and context."""
@@ -172,31 +227,31 @@ class ResourceLoader[TResource](ABC):
         return fields["resource_type"].default  # type: ignore
 
     @overload
-    async def load(
+    def load(
         self,
         context: LoaderContext[TResource],
         processor_registry: ProcessorRegistry | None = None,
-    ) -> LoadedResource: ...
+    ) -> AsyncIterator[LoadedResource]: ...
 
     @overload
-    async def load(
+    def load(
         self,
         context: TResource,
         processor_registry: ProcessorRegistry | None = None,
-    ) -> LoadedResource: ...
+    ) -> AsyncIterator[LoadedResource]: ...
 
     @overload
-    async def load(
+    def load(
         self,
         context: None = None,
         processor_registry: ProcessorRegistry | None = None,
-    ) -> LoadedResource: ...
+    ) -> AsyncIterator[LoadedResource]: ...
 
     async def load(
         self,
-        context: LoaderContext[TResource] | TResource | None = None,
+        context: LoaderContext[TResource] | None = None,
         processor_registry: ProcessorRegistry | None = None,
-    ) -> LoadedResource:
+    ) -> AsyncIterator[LoadedResource]:
         """Load and process content.
 
         Args:
@@ -226,12 +281,11 @@ class ResourceLoader[TResource](ABC):
             case _:
                 msg = f"Invalid context type: {type(context)}"
                 raise exceptions.LoaderError(msg)
-        with logfire.span(
-            "Loading resource",
-            resource_type=self.resource_type,
-            name=name,
-        ):
-            return await self._load_impl(resource, name, processor_registry)
+
+        generator = self._load_impl(resource, name, processor_registry)
+        # Then yield from the generator
+        async for result in generator:
+            yield result
 
     @abstractmethod
     async def _load_impl(
@@ -239,5 +293,6 @@ class ResourceLoader[TResource](ABC):
         resource: TResource,
         name: str,
         processor_registry: ProcessorRegistry | None,
-    ) -> LoadedResource:
+    ) -> AsyncIterator[LoadedResource]:
         """Implementation of actual loading logic."""
+        yield NotImplemented  # type: ignore
