@@ -8,11 +8,11 @@ from typing import TYPE_CHECKING, Any
 import logfire
 
 from llmling.config.models import (
+    BaseResource,
     CallableResource,
     CLIResource,
     ImageResource,
     PathResource,
-    Resource,
     SourceResource,
     TextResource,
 )
@@ -23,6 +23,8 @@ from llmling.resources.watching import ResourceWatcher
 
 
 if TYPE_CHECKING:
+    from collections.abc import AsyncIterator
+
     from llmling.processors.registry import ProcessorRegistry
     from llmling.resources.loaders.registry import ResourceLoaderRegistry
     from llmling.resources.models import LoadedResource
@@ -31,7 +33,7 @@ if TYPE_CHECKING:
 logger = get_logger(__name__)
 
 
-class ResourceRegistry(BaseRegistry[str, Resource]):
+class ResourceRegistry(BaseRegistry[str, BaseResource]):
     """Registry for managing configured resources."""
 
     def __init__(
@@ -48,9 +50,19 @@ class ResourceRegistry(BaseRegistry[str, Resource]):
         self._last_loaded: dict[str, datetime] = {}
         self.watcher = ResourceWatcher(self)
 
-    def register(self, key: str, item: Resource | Any, replace: bool = False) -> None:
+    def register(self, key: str, item: BaseResource | Any, replace: bool = False) -> None:
         """Register an item."""
-        # Existing registration logic...
+        try:
+            # Create copy with URI if needed
+            if not item.uri:
+                loader = self.loader_registry.get_loader(item)
+                uri = loader.create_uri(name=key)
+                item = item.model_copy(update={"uri": uri})
+        except exceptions.LoaderError as exc:
+            msg = f"No loader registered for resource type '{item.resource_type}'"
+            raise exceptions.ResourceError(msg) from exc
+
+        # Call parent registration
         super().register(key, item, replace)
 
         # Set up watching if needed
@@ -86,7 +98,7 @@ class ResourceRegistry(BaseRegistry[str, Resource]):
         """Error class to use for this registry."""
         return exceptions.ResourceError
 
-    def _validate_item(self, item: Any) -> Resource:
+    def _validate_item(self, item: Any) -> BaseResource:
         """Validate and possibly transform items.
 
         Args:
@@ -101,19 +113,12 @@ class ResourceRegistry(BaseRegistry[str, Resource]):
         try:
             match item:
                 # Match against concrete resource types
-                case (
-                    PathResource()
-                    | TextResource()
-                    | CLIResource()
-                    | SourceResource()
-                    | CallableResource()
-                    | ImageResource()
-                ):
+                case BaseResource():
                     return item
 
                 case dict() if "resource_type" in item:
                     # Map resource_type to appropriate class
-                    resource_classes: dict[str, type[Resource]] = {
+                    resource_classes: dict[str, type[BaseResource]] = {
                         "path": PathResource,
                         "text": TextResource,
                         "cli": CLIResource,
@@ -144,59 +149,70 @@ class ResourceRegistry(BaseRegistry[str, Resource]):
         """Get URI for a resource by name."""
         resource = self[name]
         loader = self.loader_registry.get_loader(resource)
+        loader = loader.create(resource, name)  # Create instance
         return loader.create_uri(name=name)
 
-    @logfire.instrument("Loading resource {name}")
-    async def load(self, name: str, *, force_reload: bool = False) -> LoadedResource:
-        """Load a resource by name."""
+    async def load_all(
+        self, name: str, *, force_reload: bool = False
+    ) -> AsyncIterator[LoadedResource]:
+        """Load all resources for a given name."""
         try:
             resource = self[name]
             uri = self.get_uri(name)
 
             # Check cache unless force reload
             if not force_reload and uri in self._cache:
-                return self._cache[uri]
+                yield self._cache[uri]
+                return
 
             # Get loader and initialize with context
             loader = self.loader_registry.get_loader(resource)
-            loader = loader.create(resource, name)  # Create with named context
+            loader = loader.create(resource, name)
 
-            loaded = await loader.load(
-                context=loader.context,  # Pass the context we created
+            async for loaded in loader.load(
+                context=loader.context,
                 processor_registry=self.processor_registry,
-            )
+            ):
+                # Ensure the URI is set correctly
+                if loaded.metadata.uri != uri:
+                    msg = "Loader returned different URI than expected: %s != %s"
+                    logger.warning(msg, loaded.metadata.uri, uri)
+                    loaded.metadata.uri = uri
 
-            # Ensure the URI is set correctly
-            if loaded.metadata.uri != uri:
-                logger.warning(
-                    "Loader returned different URI than expected: %s != %s",
-                    loaded.metadata.uri,
-                    uri,
-                )
-                loaded.metadata.uri = uri
+                # Update cache using URI
+                self._cache[uri] = loaded
+                self._last_loaded[uri] = datetime.now()
+
+                yield loaded
 
-            # Update cache using URI
-            self._cache[uri] = loaded
-            self._last_loaded[uri] = datetime.now()
         except KeyError as exc:
             msg = f"Resource not found: {name}"
             raise exceptions.ResourceError(msg) from exc
         except Exception as exc:
             msg = f"Failed to load resource {name}: {exc}"
             raise exceptions.ResourceError(msg) from exc
-        else:
-            return loaded
+
+    @logfire.instrument("Loading resource {name}")
+    async def load(self, name: str, *, force_reload: bool = False) -> LoadedResource:
+        """Load first/single resource (backward compatibility)."""
+        async for resource in self.load_all(name, force_reload=force_reload):
+            return resource
+        msg = f"No resources loaded for {name}"
+        raise exceptions.ResourceError(msg)
 
     async def load_by_uri(self, uri: str) -> LoadedResource:
         """Load a resource by URI."""
         try:
-            # Find loader that supports this URI
+            # For simple resource names in file:/// format
+            if uri.startswith("file:///"):
+                # Extract resource name
+                name = uri.split("/")[-1]
+                # Load using the name which will use the configured path/URL
+                return await self.load(name)
+
+            # For direct URLs or other schemes
             loader = self.loader_registry.find_loader_for_uri(uri)
-
-            # Get resource name from URI
             name = loader.get_name_from_uri(uri)
-
-            # Load resource
             return await self.load(name)
 
         except Exception as exc:
